<%= @exception.class.to_s =~ /^[aeiou]/i ? 'An' : 'A' %> <%= @exception.class %> occurred in background at <%= raw @timestamp %> :

  <%= @exception.message %>
  <%= @backtrace.first %>

<%  sections = @sections.map do |section|
      summary = render(section).strip
      unless summary.blank?
        title = render("title", :title => section).strip
        "#{title}\n\n#{summary.gsub(/^/, "  ")}\n\n"
      end
    end.join
%>
<%= raw sections %>
