<% filtered_env = @request.filtered_env -%>

<ul style="list-style: none">
  <% filtered_env.keys.map(&:to_s).sort.each do |key| -%>
    <li>
      <strong><%= key %>:</strong>
      <span><%= inspect_object(filtered_env[key]) %></span>
    </li>
  <% end -%>
</ul>
